function selection_layer = mark2selection(selection_layer, type, str_type, xy_vec)
% function selection_layer = mark2selection(selection_layer, type, str_type, xy_vec)
% Place identification symbols to the selection layer
%
% Parameters:
% selection_layer: a single slice of the selection layer, [1:height, 1:width]
% type: type of the marker: ''box'', ''circle'', ''smcross'', ''cross'', ''diamond'', ''star'', ''spot'', ''lspot'', ''sbox''
% str_type: - additional identifier of the structure: 
% - ''tube'' add a vertical line to the left-hand side of the marker
% - ''sheet'' add a vertical line to the right-hand side of the marker
% - ''unknown'' - marker without additional identifiers
% xy_vec: a vector with coordinates of the marker - xy_vec(id,x:y)
%
% Return values:
% selection_layer: modified selection layer
% @see imageData.mark2selection

% Copyright (C) 21.11.2013 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


type = [str_type type];
for ind = 1: size(xy_vec,1)
    xy = round(xy_vec(ind,:));
    % shift the window if it goes out of the image
    if xy(1)-3 < 1; xy(1) = 3+1; end;
    if xy(2)-3 < 1; xy(2) = 3+1; end;
    if xy(1)+3 > size(selection_layer,2); xy(1) = size(selection_layer,2)-3; end;
    if xy(2)+3 > size(selection_layer,1); xy(2) = size(selection_layer,1)-3; end;
    
    switch type
        case 'tuberetract'
            sel_mask = ...
                [1 0 0 0 0;...
                 1 0 0 0 0;...
                 1 0 1 1 1;...
                 1 0 0 0 0;...
                 1 0 0 0 0];
        case 'sheetretract'
            sel_mask = ...
                [0 0 0 0 1;...
                 0 0 0 0 1;...
                 1 1 1 0 1;...
                 0 0 0 0 1;...
                 0 0 0 0 1];
        case 'unknownretract'
            sel_mask = ...
                [0 0 0 0 0;...
                 0 0 0 0 0;...
                 0 1 1 1 0;...
                 0 0 0 0 0;...
                 0 0 0 0 0];
        case 'tubegrowth'
            sel_mask = ...
                [1 0 0 0 0;...
                 1 0 0 1 0;...
                 1 0 1 1 1;...
                 1 0 0 1 0;...
                 1 0 0 0 0];
        case 'sheetgrowth'
            sel_mask = ...
                [0 0 0 0 1;...
                 0 1 0 0 1;...
                 1 1 1 0 1;...
                 0 1 0 0 1;...
                 0 0 0 0 1];
        case 'unknowngrowth'
            sel_mask = ...
                [0 0 0 0 0;...
                 0 0 1 0 0;...
                 0 1 1 1 0;...
                 0 0 1 0 0;...
                 0 0 0 0 0];
        case 'tubesliding'
            sel_mask = ...
                [1 0 0 0 0;...
                 1 0 1 0 1;...
                 1 0 0 1 0;...
                 1 0 1 0 1;...
                 1 0 0 0 0];
        case 'sheetsliding'
            sel_mask = ...
                [0 0 0 0 1;...
                 1 0 1 0 1;...
                 0 1 0 0 1;...
                 1 0 1 0 1;...
                 0 0 0 0 1];
        case 'unknownsliding'
            sel_mask = ...
                [0 0 0 0 0;...
                 0 1 0 1 0;...
                 0 0 1 0 0;...
                 0 1 0 1 0;...
                 0 0 0 0 0];
        case 'cross'
            sel_mask = ...
                [0 0 0 0 0;...
                 0 0 1 0 0;...
                 0 1 1 1 0;...
                 0 0 1 0 0;...
                 0 0 0 0 0];
        case 'largecross'
            sel_mask = ...
                [0 0 1 0 0;...
                 0 0 1 0 0;...
                 1 1 1 1 1;...
                 0 0 1 0 0;...
                 0 0 1 0 0];
    end
    selection_layer(xy(2)-2:xy(2)+2, xy(1)-2:xy(1)+2) = ...
        selection_layer(xy(2)-2:xy(2)+2, xy(1)-2:xy(1)+2) | sel_mask;
    % switch str_type     % add extra info
    %     case 'tube'
    %         selection_layer(xy(2)-2:xy(2)+2, xy(1)-3:xy(1)-3) = [1; 1; 1; 1; 1];
    %     case 'sheet'
    %         selection_layer(xy(2)-2:xy(2)+2, xy(1)+3:xy(1)+3) = [1; 1; 1; 1; 1];
    %     case 'unknown'
    %
    % end
end
end